@@ -33,11 +33,13 @@ MetaDataTable = NewType('MetaDataTable', object)
 
 
 class ResourceAttribute():
-    def __init__(self, name: str, resource: Type['Resource'], type: str):
-        self.name = name
-        self.type = None  # none means not interesting, default display to simple text
-        if name == "InvoiceDate":
+    def __init__(self, each_attribute: object, resource: Type['Resource']):
+        self.name = str(each_attribute.name)
+        if self.name == "Ready":
             debug_str = "Nice breakpoint"
+        # self.nullable = each_attribute.nullable
+        type = str(each_attribute.type)
+        self.type = None  # none means not interesting, default display to simple text
         if type == "DECIMAL":
             self.type = "DECIMAL"
         elif type == "DATE":
@@ -49,7 +51,13 @@ class ResourceAttribute():
         elif type.startswith("NTEXT") == "NTEXT":
             self.type = "NTEXT"
         self.non_favorite = False
-        lower_name = name.lower()
+        self.is_required = not each_attribute.nullable
+        if self.is_required and each_attribute.primary_key:
+            if type in ["Integer", "INTEGER", "MEDIUMINT", "SMALLINT", "TINYINT"]:
+                self.is_required = False  # this is autonum... so not required (affects admin.yaml - required)
+            else:
+                debug_str = "Alpha Pkey"  # nothing to do, this for debug verification
+        lower_name = self.name.lower()
         non_favs = resource.create_from_model._non_favorite_names_list
         for each_non_fav in non_favs:
             if lower_name.endswith(each_non_fav):
@@ -152,6 +160,7 @@ class CreateFromModel(object):
 
     def __init__(self,
                  project_directory: str = "~/Desktop/my_project",
+                 api_name: str = "api",
                  copy_to_project_directory: str = "",
                  api_logic_server_dir: str = "",
                  abs_db_url: str = "sqlite:///nw.sqlite",
@@ -169,8 +178,10 @@ class CreateFromModel(object):
                  favorite_names: str = "name description",
                  non_favorite_names: str = "id",
                  command: str = "",
-                 version: str = "0.0.0"):
+                 version: str = "0.0.0",
+                 multi_api: bool = False):
         self.project_directory = None
+        self.api_name = api_name
         if project_directory:
             self.project_directory = self.get_windows_path_with_slashes(project_directory)
         self.copy_to_project_directory = ""
@@ -197,6 +208,7 @@ class CreateFromModel(object):
         self.flask_appbuilder = flask_appbuilder
         self.react_admin = react_admin
         self.version = version
+        self.multi_api = multi_api
 
         self.table_to_class_map = {}
         """ keys are table[.column], values are class / attribute """
@@ -214,10 +226,15 @@ class CreateFromModel(object):
     def get_windows_path_with_slashes(url: str) -> str:
         """ idiotic fix for windows (\ --> \\\\)
         https://stackoverflow.com/questions/1347791/unicode-error-unicodeescape-codec-cant-decode-bytes-cannot-open-text-file"""
+        """ old code
         full_path = os.path.abspath(url)
         result = full_path.replace('\\', '\\\\')
         if os.name == "nt":  # windows
             result = full_path.replace('/', '\\')
+        print(f'*** DEBUG - how about url_path={url_path}')
+        """
+        url_path = Path(url)
+        result = str(url_path)
         return result
 
     def recursive_overwrite(self, src, dest, ignore=None):
@@ -689,12 +706,26 @@ class CreateFromModel(object):
         """
         creates self.resource_list via dynamic import of models.py  (drives create_from_model modules)
         """
-        project_abs_path = abspath(self.project_directory)
+        """ old code
+        # project_abs_path = abspath(self.project_directory)
+        # project_abs_path = str(Path(self.project_directory).absolute())
+        """
+        project_path = self.project_directory
+        debug_dynamic_loader = False
+        if debug_dynamic_loader:
+            print(f'\n\n ### INSTALL cwd = {os.getcwd()}')
+            print(f'\n*** DEBUG/import - self.project_directory={self.project_directory}')
+            print(f'*** DEBUG/import - project_abs_path={project_path}')
         model_imported = False
-        path_to_add = project_abs_path if self.command == "create-ui" else \
-            project_abs_path + "/database"  # for Api Logic Server projects
-        sys.path.insert(0, path_to_add)  # e.g., adds /Users/val/Desktop/my_project/database
-        print(msg)  #    b.  Create resource_list - import database/models.py, inspect each class
+        path_to_add = project_path if self.command == "create-ui" else \
+            project_path + "/database"  # for Api Logic Server projects
+        sys.path.insert(0, self.project_directory)    # e.g., /Users/val/dev/servers/install/ApiLogicServer
+        sys.path.insert(0, path_to_add)    # e.g., /Users/val/dev/servers/install/ApiLogicServer/database
+        print(msg + " in " + path_to_add)
+        # sys.path.insert( 0, '/Users/val/dev/servers/install/ApiLogicServer/ApiLogicProject/database')
+        # sys.path.insert( 0, '/Users/val/dev/servers/install/ApiLogicServer/ApiLogicProject')  # AH HA!!
+        # sys.path.insert( 0, 'ApiLogicProject')  # or, AH HA!!
+        # print(f'*** DEBUG - sys.path={sys.path}')
         try:
             # credit: https://www.blog.pythonlibrary.org/2016/05/27/python-201-an-intro-to-importlib/
             importlib.import_module('models')
@@ -707,7 +738,7 @@ class CreateFromModel(object):
         orm_class = None
         if not model_imported:
             print('.. .. ..Creation proceeding to enable manual database/models.py fixup')
-            print('.. .. .. See https://github.com/valhuber/ApiLogicServer/wiki/Troubleshooting#manual-model-repair')
+            print('.. .. .. See https://github.com/valhuber/ApiLogicServer/wiki/Troubleshooting')
         else:
             try:
                 resource_list: Dict[str, Resource] = dict()
@@ -733,9 +764,8 @@ class CreateFromModel(object):
                         resource_data = {"type": resource_name}
                         for each_attribute in resource_class._s_columns:
                             attr_type = str(each_attribute.type)
-                            resource_attribute = ResourceAttribute(name=str(each_attribute.name),
-                                                                   resource=resource,
-                                                                   type=attr_type)
+                            resource_attribute = ResourceAttribute(each_attribute=each_attribute,
+                                                                   resource=resource)
                         for rel_name, rel in resource_class._s_relationships.items():
                             # relation = {}
                             # relation["direction"] = "toone" if rel.direction == MANYTOONE else "tomany"
@@ -834,10 +864,11 @@ class CreateFromModel(object):
         rtn_my_parents_map = None
         model_file_name = "*"
         if self.command in ('create', 'create-and-run', 'rebuild-from-database'):
-            if False and self.use_model != "":  # use-model (todo - disabled)
+            if self.use_model != "":  # use pre-existing (or repaired) existing model file
                 model_file_name = project_directory + '/database/models.py'
-                print(f' a.  Use existing {self.use_model} - copy to {project_directory + "/database/models.py"}')
-                copyfile(self.use_model, model_file_name)
+                use_model_path = Path(self.use_model).absolute()
+                print(f' a.  Use existing {use_model_path} - copy to {project_directory + "/database/models.py"}')
+                copyfile(use_model_path, model_file_name)
             else:
                 print(f' a.  Create Models - create database/models.py, using sqlcodegen for database: {abs_db_url}')
                 code_gen_args = get_codegen_args()
