@@ -4,13 +4,14 @@ import sys
 import os
 import pathlib
 from pathlib import Path
-from typing import NewType, List
+from typing import NewType, List, Tuple, Dict
 
 import sqlalchemy
 import yaml
-from sqlalchemy import MetaData
+from sqlalchemy import MetaData, false
 import datetime
-import create_from_model.model_creation_services as create_from_model
+import api_logic_server_cli.create_from_model.model_creation_services as create_from_model
+import api_logic_server_cli.create_from_model.api_logic_server_utils as create_utils
 from dotmap import DotMap
 
 from api_logic_server_cli.create_from_model.model_creation_services import Resource
@@ -98,24 +99,24 @@ class AdminCreator(object):
         self._non_favorite_names_list = self.non_favorite_names.split()
         self._favorite_names_list = self.favorite_names.split()
 
-    def create_admin_application(self) -> str:
+    def create_admin_application(self):
         """ main driver - loop through resources, write admin.yaml - with backup, nw customization
         """
         if self.mod_gen.command == "create-ui" or self.mod_gen.command.startswith("rebuild"):
             if self.mod_gen.command.startswith("rebuild"):
-                print(".. .. ..Use existing ui/admin")
+                print(".. .. ..Use existing ui/admin directory")
         else:
             self.create_admin_app(msg=".. .. ..Create ui/admin")
 
         cwd = os.getcwd()
         sys.path.append(cwd)
 
-        self.admin_yaml.api_root = f'http://localhost:5656/api'
+        self.admin_yaml.api_root = f'http://localhost:5656/{self.mod_gen.api_name}'
         if self.host != "localhost":
             if self.port !="":
-                self.admin_yaml.api_root = f'http://{self.host}:{self.port}/api'
+                self.admin_yaml.api_root = f'http://{self.host}:{self.port}/{self.mod_gen.api_name}'
             else:
-                self.admin_yaml.api_root = f'http://{self.host}/api'
+                self.admin_yaml.api_root = f'http://{self.host}/{self.mod_gen.api_name}'
         self.admin_yaml.resources = {}
         for each_resource_name in self.mod_gen.resource_list:
             each_resource = self.mod_gen.resource_list[each_resource_name]
@@ -126,11 +127,9 @@ class AdminCreator(object):
         self.create_settings()
         # self.doc_properties()
 
-        admin_yaml_dict = self.admin_yaml.toDict()
-        admin_yaml_dump = yaml.dump(admin_yaml_dict)
         if self.mod_gen.command != "create-ui":
-            self.write_yaml_files(admin_yaml_dump)
-        return admin_yaml_dump
+            self.write_yaml_files()
+
 
     def create_resource_in_admin(self, resource: Resource):
         """ self.admin_yaml.resources += resource DotMap for given resource
@@ -148,7 +147,7 @@ class AdminCreator(object):
                 new_resource.tab_groups = child_tabs
             self.admin_yaml.resources[resource.table_name] = new_resource.toDict()
 
-    def create_attributes_in_owner(self, owner: DotMap, resource: Resource, owner_resource: (None, Resource)):
+    def create_attributes_in_owner(self, owner: DotMap, resource: Resource, owner_resource) -> Dict[None, Resource]:
         """ create attributes in owner (owner is a DotMap -- of resource, or tab)
 
             Order:
@@ -182,8 +181,15 @@ class AdminCreator(object):
             if admin_parent_joins_implicit:  # temp hack - just do the FK
                 fk_pair = each_parent.parent_child_key_pairs[0]  # assume single-field keys
                 fk_attr_name = fk_pair[1]
+                resource_attribute = None
+                for each_attribute in resource.attributes:
+                    if each_attribute.name == fk_attr_name:
+                        resource_attribute = each_attribute
+                        break
+                if resource_attribute is None:
+                    raise Exception(f'System Error: unable to find {fk_attr_name} in {resource.name}')
                 processed_attributes.add(fk_attr_name)
-                admin_attribute = self.create_admin_attribute(fk_attr_name)
+                admin_attribute = self.create_admin_attribute(resource_attribute)
                 if admin_attribute is not None:
                     attributes_dict.append(admin_attribute)
             else:
@@ -221,17 +227,25 @@ class AdminCreator(object):
 
     @staticmethod
     def create_admin_attribute(resource_attribute) -> DotMap:
+        """ create attribute entry for admin.yaml
+        """
         attribute_name = resource_attribute if isinstance(resource_attribute, str) else resource_attribute.name
+        required = False if isinstance(resource_attribute, str) else resource_attribute.is_required
         admin_attribute = DotMap()
         admin_attribute.name = str(attribute_name)
-        if attribute_name == "InvoiceDate":
+        if required:
+            admin_attribute.required = True
+        if attribute_name == "xShippedDate":
             log.debug("Good breakpoint location")
+        if isinstance(resource_attribute, str) == True:
+            log.debug("Just a string")
+            raise Exception(f'System Error - expected resource_attribute, got string: {resource_attribute}')
         if not isinstance(resource_attribute, str):
             if resource_attribute.type in ["DECIMAL", "DATE"]:
                 admin_attribute.type = resource_attribute.type
             if resource_attribute.type in ["NTEXT", "IMAGE"]:
                 admin_attribute = None
-        return admin_attribute
+        return admin_attribute  #.toDict()  hmm... sometimes a "shape" property slips in...?
 
     @staticmethod
     def cap_space(text):
@@ -242,46 +256,8 @@ class AdminCreator(object):
             new_text += letter
         return new_text
 
-    def create_attributes_in_owner_zz(self, owner: DotMap, resource: Resource, owner_resource: (None, Resource)):
-        """ create attributes in owner (owner is a DotMap -- of resource, or tab)
-
-          Customer:
-            attributes:
-            - label: CompanyName*
-              name: CompanyName
-              search: true
-            - name: ContactName
-            - name: ContactTitle
-        """
-        owner.attributes = []
-        attributes_dict = []  # DotMap()
-        if admin_attr_ordering:
-            attributes = self.mod_gen.get_show_attributes(resource)
-        else:
-            attributes = self.mod_gen.get_attributes(resource)
-        for each_attribute in attributes:
-            if "." not in each_attribute:   # not a parent join
-                admin_attribute = DotMap()
-                admin_attribute.name = each_attribute
-                if each_attribute == self.mod_gen.favorite_attribute_name(resource):
-                    admin_attribute.search = True
-                    admin_attribute.label = f"{each_attribute}*"
-                """
-                if each_attribute.type in ["DECIMAL", "DATE"]:
-                    admin_attribute.type = each_attribute.type
-                """
-                attributes_dict.append(admin_attribute)
-            else:                           # parent join (disabled code - overwritten at end)
-                relationship = self.new_relationship_to_parent(resource, each_attribute, owner_resource)
-                if relationship is not None:  # skip redundant master join
-                    rel = DotMap()
-                    parent_role_name = each_attribute.split('.')[0]
-                    rel[parent_role_name] = relationship.toDict()
-                    owner.attributes.append(rel)
-        owner.attributes = attributes_dict  # for now, just attrs, no parent joins since safrs-react autojoins FKs
-
     def new_relationship_to_parent(self, a_child_resource: Resource, parent_attribute_reference,
-                                      a_master_parent_resource) -> (None, DotMap):
+                                      a_master_parent_resource) -> DotMap:
         """
         given a_child_table_def.parent_column_reference, create relationship: attrs, fKeys (for *js* client (no meta))
 
@@ -300,6 +276,7 @@ class AdminCreator(object):
         :param a_child_resource: a child resource (not class), eg, Employees
         :param parent_attribute_reference: parent ref, eg, Department1.DepartmentName
         :param a_master_parent_resource: the master of master/detail - skip joins for this
+        :returns DotMap relationship
         """
         parent_role_name = parent_attribute_reference.split('.')[0]  # careful - is role (class) name, not table name
         if a_master_parent_resource is not None and parent_role_name == a_master_parent_resource.name:
@@ -450,7 +427,7 @@ class AdminCreator(object):
         return tab_group
 
     def new_relationship_to_parent_no_model(self, a_child_table_def: MetaDataTable, parent_column_reference,
-                                   a_master_parent_table_def) -> (None, DotMap):
+                                   a_master_parent_table_def) -> DotMap:
         """
         Rarely used, now broken.  Ignore for now.
 
@@ -502,39 +479,125 @@ class AdminCreator(object):
         parent_path = parent_path.parent
         return parent_path
 
-    def write_yaml_files(self, admin_yaml):
-        """ write admin.yaml, with backup, with additional nw customized backup
+    def write_yaml_files(self):
         """
-        yaml_file_name = self.mod_gen.fix_win_path(self.mod_gen.project_directory + f'/ui/admin/admin.yaml')
-        """  todo - overwrite if never altered
-        import os.path, time
-        print("Last modified: %s" % time.ctime(os.path.getmtime("test.txt")))
-        print("Created: %s" % time.ctime(os.path.getctime("test.txt")))
+        write admin.yaml from self.admin_yaml.toDict()
+
+        with -created backup, with additional nw customized backup
         """
-        if not self.mod_gen.command.startswith("rebuild"):
+        admin_yaml_dict = self.admin_yaml.toDict()
+        admin_yaml_dump = yaml.dump(admin_yaml_dict)
+
+        yaml_file_name = os.path.join(Path(self.mod_gen.project_directory), Path(f'ui/admin/admin.yaml'))
+        write_file = "Write"
+        if self.mod_gen.command.startswith("rebuild"):
+            write_file = "Rebuild - preserve"
+
+        ''' is not working on mac - always appears unaltered
+            https://stackoverflow.com/questions/946967/get-file-creation-time-with-python-on-mac
+
+        enable_rebuild_unaltered = False        
+        write_file = "Write"
+        if self.mod_gen.command.startswith("rebuild"):
+            write_file = "Rebuild - preserve"
+            created_time = os.path.getctime(yaml_file_name)
+            birth_time = os.stat(yaml_file_name).st_birthtime
+            modified_time = os.path.getmtime(yaml_file_name)
+            if enable_rebuild_unaltered and created_time == modified_time:
+                write_file = "Rebuild - overwrite unaltered"
+        '''
+        if write_file == "Rebuild - preserve":
+            yaml_merge_file_name = os.path.join(Path(self.mod_gen.project_directory), Path(f'ui/admin/admin-merge.yaml'))
+            print(f'.. .. ..{write_file} {yaml_file_name} - creating merge at {yaml_merge_file_name}')
+            merge_yaml = self.create_yaml_merge()
+            admin_merge_yaml_dump = yaml.dump(merge_yaml)
+            with open(yaml_merge_file_name, 'w') as yaml_merge_file:
+                yaml_merge_file.write(admin_merge_yaml_dump)
+        else:
+            print(f'.. .. ..{write_file} {yaml_file_name}')
             with open(yaml_file_name, 'w') as yaml_file:
-                yaml_file.write(admin_yaml)
+                yaml_file.write(admin_yaml_dump)
 
         yaml_created_file_name = \
-            self.mod_gen.fix_win_path(self.mod_gen.project_directory + f'/ui/admin/admin-created.yaml')
-        with open(yaml_created_file_name, 'w') as yaml_copy_file:
-            yaml_copy_file.write(admin_yaml)
+            os.path.join(Path(self.mod_gen.project_directory), Path(f'ui/admin/admin-created.yaml'))
+        with open(yaml_created_file_name, 'w') as yaml_created_file:
+            yaml_created_file.write(admin_yaml_dump)
 
-        if self.mod_gen.nw_db_status in ["nw", "nw-"]:
-            admin_custom_nw_file = open(
-                os.path.dirname(os.path.realpath(__file__)) + "/templates/admin_custom_nw.yaml")
-            admin_custom_nw = admin_custom_nw_file.read()
-            nw_backup_file_name = \
-                self.mod_gen.fix_win_path(self.mod_gen.project_directory + f'/ui/admin/admin_custom_nw.yaml')
-            admin_file = open(nw_backup_file_name, 'w')
-            admin_file.write(admin_custom_nw)
-            admin_file.close()
-            dev_temp_do_not_overwrite = False
-            if not dev_temp_do_not_overwrite:
-                print('.. .. ..Using customized admin_custom_nw.yaml - compare to admin-created.yaml')
-                admin_file = open(yaml_file_name, 'w')
+        if self.mod_gen.nw_db_status in ["nw", "nw-"] and self.mod_gen.api_name == "api":
+            if not self.mod_gen.command.startswith("rebuild"):
+                pass
+                src = os.path.join(Path(self.mod_gen.project_directory), Path(f'ui/admin/admin_custom_nw.yaml'))
+                dest = os.path.join(Path(self.mod_gen.project_directory), Path(f'ui/admin/admin.yaml'))
+                shutil.copyfile(src, dest)
+                """  nw fixup (not required, since all copied from prototype_nw)
+                admin_custom_nw_file = open(
+                    os.path.dirname(os.path.realpath(__file__)) + "/templates/admin_custom_nw.yaml")
+                admin_custom_nw = admin_custom_nw_file.read()
+                nw_backup_file_name = \
+                    os.path.join(Path(self.mod_gen.project_directory), Path(f'ui/admin/admin_custom_nw.yaml'))
+                admin_file = open(nw_backup_file_name, 'w')
                 admin_file.write(admin_custom_nw)
                 admin_file.close()
+                dev_temp_do_not_overwrite = False
+                if not dev_temp_do_not_overwrite:
+                    print('.. .. ..Using customized admin_custom_nw.yaml - compare to admin-created.yaml')
+                    admin_file = open(yaml_file_name, 'w')
+                    admin_file.write(admin_custom_nw)
+                    admin_file.close()
+                """
+
+    def create_yaml_merge(self) -> dict:
+        """
+        return admin_merge.yaml from self.admin_yaml.toDict() and ui/admin/admin.yaml
+        xxx
+        """
+        yaml_admin_file_name = \
+            os.path.join(Path(self.mod_gen.project_directory), Path(f'ui/admin/admin.yaml'))
+        with open(yaml_admin_file_name,'r')as file_descriptor: # new rsc, attr
+            merge_yaml_dict = yaml.load(file_descriptor, Loader=yaml.SafeLoader)
+        merge_resources = merge_yaml_dict['resources']
+        current_resources = self.admin_yaml.resources
+        new_resources = ''
+        new_attributes = ''
+        new_tab_groups = ''
+        for each_resource_name, each_resource in current_resources.items():
+            if each_resource_name not in merge_resources:
+                new_resources = new_resources + f'{each_resource_name} '
+                merge_resources[each_resource_name] = each_resource
+            else:
+                current_attributes = each_resource['attributes']
+                merge_attributes = merge_resources[each_resource_name]['attributes']
+                for each_current_attribute in current_attributes:
+                    attribute_name = each_current_attribute['name']
+                    attribute_found = False
+                    for each_merge_attribute in merge_attributes:
+                        if attribute_name == each_merge_attribute['name']:
+                            attribute_found = True
+                            break
+                    if not attribute_found:
+                        new_attributes = new_attributes + f'{each_resource_name}.{attribute_name} '
+                        merge_attributes.append(each_current_attribute)
+                if 'tab_groups' in each_resource:
+                    current_tab_groups = each_resource['tab_groups']
+                    if 'tab_groups' not in merge_resources[each_resource_name]:
+                        merge_resources[each_resource_name]['tab_groups'] = []
+                    merge_tab_groups = merge_resources[each_resource_name]['tab_groups']
+                    for each_current_tab_group in current_tab_groups:
+                        tab_group_name = each_current_tab_group['name']
+                        tab_group_found = False
+                        for each_merge_tab_group in merge_tab_groups:
+                            if tab_group_name == each_merge_tab_group['name']:
+                                tab_group_found = True
+                                break
+                        if not tab_group_found:
+                            new_tab_groups = new_tab_groups +f'{each_resource_name}.{tab_group_name} '
+                            merge_tab_groups.append(each_current_tab_group)
+        merge_yaml_dict['about']['merged'] = {}
+        merge_yaml_dict['about']['merged']['at'] = str(datetime.datetime.now().strftime("%B %d, %Y %H:%M:%S"))
+        merge_yaml_dict['about']['merged']['new_resources'] = new_resources
+        merge_yaml_dict['about']['merged']['new_attributes'] = new_attributes
+        merge_yaml_dict['about']['merged']['new_tab_groups'] = new_tab_groups
+        return merge_yaml_dict
 
     def create_settings(self):
         self.admin_yaml.settings = DotMap()
@@ -606,22 +669,31 @@ class AdminCreator(object):
             from_proto_dir = pathlib.Path(self.get_create_from_model_dir()).\
                 joinpath("create_from_model", "safrs-react-admin-npm-build")
         to_project_dir = pathlib.Path(self.mod_gen.project_directory).joinpath("ui", "safrs-react-admin")
-        print(f'{msg} copy prototype admin project {from_proto_dir} -> {to_project_dir}')
-        if not os.path.isdir(from_proto_dir):
-            print(f'\n==> Error - prototype admin project... did you complete setup: https://github.com/valhuber/ApiLogicServer/wiki/Internals')
-            exit(1)
-        shutil.copytree(from_proto_dir, to_project_dir)
+        use_alsdock_sra = True
+        '''
+        set False if alsdock/dockerfile copies this folder:
+        RUN cp -r /app/ui/safrs-react-admin /app/ApiLogicServer-main/api_logic_server_cli/create_from_model/safrs-react-admin-npm-build
+        '''
+        if use_alsdock_sra and self.mod_gen.multi_api:
+            print(f'{msg} multi_api - copy safrs-react-admin {from_proto_dir} -> {to_project_dir}')
+            from_proto_dir = pathlib.Path("/app/ui/safrs-react-admin")  # enables debug for alsdock projects
+            shutil.copytree(from_proto_dir, to_project_dir)
+        else:
+            print(f'{msg} copy safrs-react-admin: {from_proto_dir} -> {to_project_dir}')
+            if not os.path.isdir(from_proto_dir):
+                print(f'\n==> Error - safrs-react-admin... did you complete setup: https://github.com/valhuber/ApiLogicServer/wiki/Internals')
+                print(".. Setup required.  Really.")
+                exit(1)
+            shutil.copytree(from_proto_dir, to_project_dir)
 
-        os.mkdir(pathlib.Path(self.mod_gen.project_directory).joinpath("ui", "admin"))
-
-        home_js_file = "home.js"  # get the welcome screen
-        if self.mod_gen.nw_db_status in ["nw", "nw+"]:  # explains nw-specific customizations
-            home_js_file = "home_nw.js"
-        home_js = pathlib.Path(self.get_create_from_model_dir()).\
-            joinpath("create_from_model", "templates", home_js_file)
         to_project_dir = pathlib.Path(self.mod_gen.project_directory).joinpath("ui", "admin")
-        shutil.copyfile(home_js, to_project_dir.joinpath("home.js"))
-
+        swagger_name = self.mod_gen.api_name
+        if self.mod_gen.multi_api:
+            swagger_name += "/api"
+            print(f'.. ui/admin/home.js updated url: {swagger_name}')
+        create_utils.replace_string_in_file(search_for="api_logic_server_api_name",  # last node of server url
+                                        replace_with=swagger_name,
+                                        in_file=to_project_dir.joinpath("home.js"))
 
 def create(model_creation_services: create_from_model.CreateFromModel):
     """ called by ApiLogicServer CLI -- creates ui/admin application (ui/admin folder, admin.yaml)
@@ -631,5 +703,5 @@ def create(model_creation_services: create_from_model.CreateFromModel):
                                  not_exposed=model_creation_services.not_exposed + " ",
                                  favorite_names=model_creation_services.favorite_names,
                                  non_favorite_names=model_creation_services.non_favorite_names)
-    return admin_creator.create_admin_application()
+    admin_creator.create_admin_application()
 
