@@ -189,8 +189,13 @@ class Model(object):
             name = '_' + name
         elif name == 'metadata':
             name = 'metadata_'
+        name = name.replace("$", "_S_")   # ApiLogicServer valid name fixes for superclass version (why override?)
+        name = name.replace(" ", "_")
+        name = name.replace("+", "_")
+        name = name.replace("-", "_")
 
-        return _re_invalid_identifier.sub('_', name)
+        result = _re_invalid_identifier.sub('_', name)
+        return result
 
 
 class ModelTable(Model):
@@ -248,9 +253,17 @@ class ModelClass(Model):
 
     @classmethod
     def _tablename_to_classname(cls, tablename, inflect_engine):
+        """
+        camel-case and singlularize, with provisions for reserved word (Date) and collisions (Dates & _Dates)
+        """
         tablename = cls._convert_to_valid_identifier(tablename)
+        if tablename in ["Dates"]:  # ApiLogicServer
+            tablename = tablename + "Classs"
         camel_case_name = ''.join(part[:1].upper() + part[1:] for part in tablename.split('_'))
-        return inflect_engine.singular_noun(camel_case_name) or camel_case_name
+        if camel_case_name in ["Dates"]:
+            camel_case_name = camel_case_name + "_Classs"
+        result = inflect_engine.singular_noun(camel_case_name) or camel_case_name
+        return result
 
     @staticmethod
     def _convert_to_valid_identifier(name):  # TODO review
@@ -259,8 +272,13 @@ class ModelClass(Model):
             name = "_" + name
         elif name == "metadata":
             name = "metadata_"
+        name = name.replace("$", "_S_")   # ApiLogicServer valid name fixes, ModelClass version
+        name = name.replace(" ", "_")
+        name = name.replace("+", "_")
+        name = name.replace("-", "_")
 
-        return _re_invalid_identifier.sub("_", name)
+        result = _re_invalid_identifier.sub("_", name)
+        return result
 
     def _add_attribute(self, attrname, value):
         """ add table column/relationship to attributes
@@ -478,6 +496,8 @@ class CodeGenerator(object):
 
             # Only form model classes for tables that have a primary key and are not association
             # tables
+            if "productvariantsoh-20190423" in (table.name + ""):
+                debug_str = "target table located"
             if noclasses or not table.primary_key or table.name in association_tables:
                 model = self.table_model(table)
             else:
@@ -521,16 +541,14 @@ class CodeGenerator(object):
 from safrs import SAFRSBase
 
 import safrs
-db = safrs.DB
 
-Base = db.Model
+Base = declarative_base()
 metadata = Base.metadata
 
-NullType = db.String  # datatype fixup
-TIMESTAMP= db.TIMESTAMP
+#NullType = db.String  # datatype fixup
+#TIMESTAMP= db.TIMESTAMP
 
 from sqlalchemy.dialects.mysql import *
-
 ########################################################################################################################
 """
             # return 'Base = declarative_base()\nmetadata = Base.metadata'
@@ -666,23 +684,39 @@ from sqlalchemy.dialects.mysql import *
                 persist_arg = ', persisted={}'.format(column.server_default.persisted)
 
             server_default = 'Computed({!r}{})'.format(expression, persist_arg)
+
         elif column.server_default:
-            # The quote escaping does not cover pathological cases but should mostly work
-            default_expr = self._get_compiled_expression(column.server_default.arg)
-            if '\n' in default_expr:
-                server_default = 'server_default=text("""\\\n{0}""")'.format(default_expr)
+            # The quote escaping does not cover pathological cases but should mostly work FIXME SqlSvr no .arg
+            # not used for postgres/mysql; for sqlite, text is '0'
+            if not hasattr( column.server_default, 'arg' ):
+                server_default = 'server_default=text("{0}")'.format('0')
             else:
-                default_expr = default_expr.replace('"', '\\"')
-                server_default = 'server_default=text("{0}")'.format(default_expr)
+                default_expr = self._get_compiled_expression(column.server_default.arg)
+                if '\n' in default_expr:
+                    server_default = 'server_default=text("""\\\n{0}""")'.format(default_expr)
+                else:
+                    default_expr = default_expr.replace('"', '\\"')
+                    server_default = 'server_default=text("{0}")'.format(default_expr)
 
         comment = getattr(column, 'comment', None)
+        if (column.name + "") == "debug_column_name":
+            db = 'Column({0})'.format(', '.join(
+            ([repr(column.name)] if show_name else []) +
+            ([self.render_column_type(column.type)] if render_coltype else []) +
+            [self.render_constraint(x) for x in dedicated_fks] +
+            [repr(x) for x in column.constraints] +
+            ([server_default] if server_default else []) +
+            ['{0}={1}'.format(k, repr(getattr(column, k))) for k in kwarg] +
+            (['comment={!r}'.format(comment)] if comment and not self.nocomments else [])
+            ))
+            print("Debug Stop")  # ApiLogicServer fix for putting this at end:  index=True
         return 'Column({0})'.format(', '.join(
             ([repr(column.name)] if show_name else []) +
             ([self.render_column_type(column.type)] if render_coltype else []) +
             [self.render_constraint(x) for x in dedicated_fks] +
             [repr(x) for x in column.constraints] +
-            ['{0}={1}'.format(k, repr(getattr(column, k))) for k in kwarg] +
             ([server_default] if server_default else []) +
+            ['{0}={1}'.format(k, repr(getattr(column, k))) for k in kwarg] +
             (['comment={!r}'.format(comment)] if comment and not self.nocomments else [])
         ))
 
@@ -720,10 +754,16 @@ from sqlalchemy.dialects.mysql import *
 
     def render_table(self, model):
         # Manual edit:
-        # replace invalid chars  TODO review ApiLogicServer
-        table_name = model.table.name.replace("$", "_S_")
+        # replace invalid chars for views etc  TODO review ApiLogicServer -- using model.name vs model.table.name
+        table_name = model.name
+        bad_chars = r"$-+ "
+        if any(elem in table_name for elem in bad_chars):
+            print("sys error")
+
+        table_name = table_name.replace("$", "_S_")
         table_name = table_name.replace(" ", "_")
-        if model.table.name == "Alphabetical list of products DEBUG ONLY":
+        table_name = table_name.replace("+", "_")
+        if model.table.name == "Plus+Table":
             print("Debug Stop on table")
         rendered = "t_{0} = Table(\n{1}{0!r}, metadata,\n".format(table_name, self.indentation)
 
