@@ -1,7 +1,11 @@
 from safrs import SAFRSAPI
 import safrs
-from database import models
+import importlib
+import pathlib
 import logging as logging
 
+# use absolute path import for easier multi-{app,model,db} support
+database = __import__('database')
+
 app_logger = logging.getLogger('api_logic_server_app')
 app_logger.info("api/expose_api_models.py - endpoint for each table")
